import java.io.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * Clase que realiza el ataque de cumpleaños
 * @author Jesús Vidal Panalés
 *
 */
public class AtaqueHash extends Thread
{
	private int rondas;
	private String archivo_original;
	private String archivo_falso;
	private File ruta_originales;
	private File ruta_falsos;
	private Vector<Vector<String>> resultados;
	private JTextPane areaResultados;
	private JTextPane areaProceso;
	private JPanel panelCombo;
	private JComboBox comboOriginales;
	private JComboBox comboFalsos;
	private JFrame ataqueFrame;
	private SimpleAttributeSet attrs;
	private Hashtable<String, String> hashMensajesOriginales;
	private Hashtable<String, String> hashMensajesFalsos;
	
	/**
	 * Clase que crea una ventana para interactuar con el proceso del ataque.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class FrameResultadoAtaque
	{
		/**
		 * ActionListener para Cerrar la ventana y liberarla de la memoria.
		 * @author Jesús Vidal Panalés
		 *
		 */
		private class Cerrar implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				JButton cerrar = (JButton) e.getSource();
				JFrame jc = (JFrame)cerrar.getParent().getParent().getParent().getParent().getParent();
				jc.dispose();
			}
		}

		/**
		 * ActionListener para abrir un mensaje original y mensaje falso de mismo hash
		 * seleccionados en el combobox
		 * @author Jesús Vidal Panalés
		 *
		 */
		private class Abrir implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				if (archivo_original.equals("")||archivo_falso.equals(""))
					JOptionPane.showMessageDialog(null, "No hay mensajes para abrir", "Error", JOptionPane.ERROR_MESSAGE);
				else
				{
					File original = new File (ruta_originales.getAbsolutePath()+File.separator+archivo_original);
					File falso = new File (ruta_falsos.getAbsolutePath()+File.separator+archivo_falso);
					new AbrirMensajes(original, falso);
				}
			}
		}

		/**
		 * Constructor del Frame. Se creará la ventana donde se mostrará
		 * el proceso del ataque y los resultados.
		 *
		 */
		public FrameResultadoAtaque()
		{
			JLabel informacion = new JLabel("El proceso puede durar bastante tiempo, tenga paciencia.");
			informacion.setPreferredSize(new Dimension (700, 25));
			informacion.setSize(new Dimension (700, 25));
			informacion.setFont(new Font("Arial", Font.BOLD, 14));
			informacion.setForeground(Color.white);
			informacion.setHorizontalTextPosition(SwingConstants.CENTER);
			informacion.setHorizontalAlignment(SwingConstants.CENTER);
			
			JLabel etiquetaResultados = new JLabel("Resultados");
			etiquetaResultados.setPreferredSize(new Dimension (430,25));
			etiquetaResultados.setHorizontalTextPosition(SwingConstants.CENTER);
			etiquetaResultados.setHorizontalAlignment(SwingConstants.CENTER);
			etiquetaResultados.setBorder(BorderFactory.createLineBorder(Color.white));
			etiquetaResultados.setForeground(Color.white);
			
			JLabel etiquetaProceso = new JLabel("Proceso");
			etiquetaProceso.setPreferredSize(new Dimension (330,25));
			etiquetaProceso.setHorizontalTextPosition(SwingConstants.CENTER);
			etiquetaProceso.setHorizontalAlignment(SwingConstants.CENTER);
			etiquetaProceso.setBorder(BorderFactory.createLineBorder(Color.white));
			etiquetaProceso.setForeground(Color.white);

			JScrollPane scrollResultados = new JScrollPane(areaResultados);
			scrollResultados.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			scrollResultados.setPreferredSize(new Dimension(430, 250));
			scrollResultados.setBorder
	        (
	        		BorderFactory.createCompoundBorder
	        		(
	        				BorderFactory.createEmptyBorder(5,5,5,5),
	        				scrollResultados.getBorder()
	                )
	        );
			scrollResultados.setBackground(Color.black);

			JScrollPane scrollProceso = new JScrollPane(areaProceso);
			scrollProceso.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			scrollProceso.setPreferredSize(new Dimension(330, 250));
			scrollProceso.setBorder
	        (
	        		BorderFactory.createCompoundBorder
	        		(
	        				BorderFactory.createEmptyBorder(5,5,5,5),
	        				scrollProceso.getBorder()
	                )
	        );
			scrollProceso.setBackground(Color.black);
			
			panelCombo = new JPanel();
			panelCombo.setLayout(new FlowLayout());
			panelCombo.setSize(new Dimension(800,50));
			panelCombo.setPreferredSize(new Dimension(800,50));
			panelCombo.setBackground(Color.black);

			JLabel etiquetaOriginales = new JLabel ("Originales: ");
			etiquetaOriginales.setForeground(Color.white);
			JLabel etiquetaFalsos = new JLabel ("Falsos: ");
			etiquetaFalsos.setForeground(Color.white);
			comboFalsos = new JComboBox();
			comboFalsos.setPreferredSize(new Dimension(120,20));
			comboFalsos.addActionListener(new SeleccionarFalso());
			
			comboOriginales = new JComboBox();
			comboOriginales.setPreferredSize(new Dimension(120,20));
			comboOriginales.addActionListener(new SeleccionarOriginal());

			ImageIcon botonAbrirNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonAbrirNormal.png"));
			ImageIcon botonAbrirPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonAbrirPulsado.png"));
			Dimension boton = new Dimension (botonAbrirNormal.getIconWidth(), botonAbrirNormal.getIconHeight());
			
			JButton botonAbrir = new JButton();
			botonAbrir.setIcon(botonAbrirNormal);
			botonAbrir.addActionListener(new Abrir());
			botonAbrir.setToolTipText("Abre los ficheros original y falsos seleccionados");
			botonAbrir.setPreferredSize(boton);
			botonAbrir.setPressedIcon(botonAbrirPulsado);
			botonAbrir.setBorderPainted(false);
			
			panelCombo.add(etiquetaOriginales);
			panelCombo.add(comboOriginales);
			panelCombo.add(etiquetaFalsos);
			panelCombo.add(comboFalsos);
			panelCombo.add(botonAbrir);
			panelCombo.validate();
			
			ImageIcon botonCerrarNormal = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarNormal.png"));
			ImageIcon botonCerrarPulsado = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarPulsado.png"));

			JButton botonCerrar = new JButton();
			botonCerrar.setIcon(botonCerrarNormal);
			botonCerrar.addActionListener(new Cerrar());
			botonCerrar.setToolTipText("Cierra la ventana");
			botonCerrar.setPreferredSize(boton);
			botonCerrar.setPressedIcon(botonCerrarPulsado);
			botonCerrar.setBorderPainted(false);
			
			JPanel panel = new JPanel();
			panel.setPreferredSize(new Dimension (800,460));
			panel.add(informacion);
			panel.add(etiquetaProceso);
			panel.add(etiquetaResultados);
			panel.add(scrollProceso);
			panel.add(scrollResultados);
			panel.add(panelCombo);
			panel.add(botonCerrar);
			panel.setBackground(Color.black);
			
			ataqueFrame.setBackground(Color.black);
			ataqueFrame.setTitle("Resultado del ataque");
			ataqueFrame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			ataqueFrame.setPreferredSize(new Dimension (800,460));
			ataqueFrame.setSize(new Dimension (800,460));
			ataqueFrame.setResizable(false);
			ataqueFrame.getContentPane().add(panel);
			ataqueFrame.setIconImage(new ImageIcon (getClass().getClassLoader().getResource("images/iconoHash.png")).getImage());
			ataqueFrame.pack();
			ataqueFrame.setVisible(true);
		}
	}
	
	/**
	 * Constructor del proceso del ataque de cumpleaños
	 * @param ruta_originales Ruta de los archivos originales
	 * @param ruta_falsos Ruta de los archivos falsos para encontrar un mismo hash que un mensaje original
	 * @param r Las rondas para el proceso de SDES
	 */
	public AtaqueHash (File ruta_originales, File ruta_falsos, int r)
	{
		this.ruta_falsos = ruta_falsos;
		this.ruta_originales = ruta_originales;
		this.rondas = r;
		this.hashMensajesOriginales = new Hashtable<String, String>();
		this.hashMensajesFalsos = new Hashtable<String, String>();

		ataqueFrame = new JFrame();
		
		attrs = new SimpleAttributeSet();
		StyleConstants.setBold(attrs, true);
		
		areaResultados = new JTextPane();
		areaResultados.setPreferredSize(new Dimension (250,250));
		areaProceso = new JTextPane();
		areaProceso.setPreferredSize(new Dimension (250,250));
		try
		{
			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), "Procesando...\n\n", attrs);
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), "Procesando...\n", attrs);
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
		areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());
		areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
		
		new FrameResultadoAtaque();
	}
	
	/**
	 * Método necesario para la ejecución del hilo del ataque de cumpleaños.
	 */
	public void run ()
	{
		StyleConstants.setBold(attrs, true);
		StyleConstants.setForeground(attrs, Color.black);
		if (this.atacar())
		{
			actualizarComboBox();
			JOptionPane.showMessageDialog(null, "Finalizó el proceso con "+resultados.size()+(resultados.size() > 1 ? " mensajes ":" mensaje ")+ "con coincidencias en mensajes falsos.", "El ataque acabó el proceso", JOptionPane.INFORMATION_MESSAGE);
		}
	}

	/**
	 * Método que realiza el ataque de cumpleaños.
	 * @return True en caso de haber encontrado algún mensaje falso con el mismo hash
	 */
	private boolean atacar()
	{
		Hash hash;
		String texto="";
		File []mensajesOriginales = ruta_originales.listFiles();
		File []mensajesFalsos = ruta_falsos.listFiles();
		Vector<String> nombreMensajesOriginales = new Vector<String>();
		Vector<String> nombreMensajesFalsos = new Vector<String>();
		this.resultados = new Vector<Vector<String>>();
		
		BufferedReader mensajeOriginal;
		BufferedReader mensajeFalso;
		String cadena = "";
		try
		{
			StyleConstants.setBold(attrs, true);
    		StyleConstants.setForeground(attrs, Color.red);
    		cadena = "\nMensajes originales\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);

			for (int i = 0; i< mensajesOriginales.length; i++)
	 		{
 				if (mensajesOriginales[i].isFile())
				{
					mensajeOriginal = new BufferedReader(new FileReader(mensajesOriginales[i]));
					String mensaje = "";

		    		while ((texto = mensajeOriginal.readLine()) != null)
			    	{
			    		mensaje += texto;	
			    	}
			    	hash = new Hash (mensaje, this.rondas);
			    	String resultado_hash = hash.funcion_hash();
			    	this.hashMensajesOriginales.put(mensajesOriginales[i].getName(), resultado_hash);
			    	nombreMensajesOriginales.add(mensajesOriginales[i].getName());
				    	
			    	//muestro la generacion de hash en el mismo areaProceso
			    	StyleConstants.setBold(attrs, true);
		    		StyleConstants.setForeground(attrs, Color.blue);
		    		cadena = mensajesOriginales[i].getName()+": ";
	    			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			    	StyleConstants.setBold(attrs, false);
		    		StyleConstants.setForeground(attrs, Color.black);
		    		cadena = resultado_hash+"\n";
	    			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				}
 				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
	 		}
			
			StyleConstants.setBold(attrs, true);
    		StyleConstants.setForeground(attrs, Color.red);
    		cadena = "\nMensajes falsos\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);

			for (int i = 0; i< mensajesFalsos.length; i++)
	 		{
 				if (mensajesFalsos[i].isFile())
				{
					mensajeFalso = new BufferedReader(new FileReader(mensajesFalsos[i]));
					String mensaje = "";
	
		    		while ((texto = mensajeFalso.readLine()) != null)
			    	{
			    		mensaje += texto;	
			    	}
			    	hash = new Hash (mensaje, this.rondas);
			    	String resultado_hash = hash.funcion_hash();
			    	this.hashMensajesFalsos.put(mensajesFalsos[i].getName(), resultado_hash);
			    	nombreMensajesFalsos.add(mensajesFalsos[i].getName());
			    	//muestro la generacion de hash en el mismo areaProceso
			    	StyleConstants.setBold(attrs, true);
		    		StyleConstants.setForeground(attrs, Color.blue);
		    		cadena = mensajesFalsos[i].getName()+": ";
	    			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			    	StyleConstants.setBold(attrs, false);
		    		StyleConstants.setForeground(attrs, Color.black);
		    		cadena = resultado_hash+"\n";
	    			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				}
 				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
	 		}
		}
		catch (BadLocationException ble)
    	{
    		ble.printStackTrace();
    	}
		catch (FileNotFoundException fnfe)
		{
			fnfe.printStackTrace();
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace();
		}

		try
		{
			System.gc();
			JOptionPane.showMessageDialog(null, "Se generaron los hash de los mensajes originales y falsos,\npulse aceptar para comenzar el ataque.", "Hash generados", JOptionPane.INFORMATION_MESSAGE);
			sleep (500);
			for (int i=0; i<nombreMensajesOriginales.size(); i++)
			{
	    		StyleConstants.setBold(attrs, true);
				StyleConstants.setForeground(attrs, Color.red);
				cadena = "\nOriginal: "+ nombreMensajesOriginales.elementAt(i)+" - Hash: "+hashMensajesOriginales.get(nombreMensajesOriginales.elementAt(i))+"\n";
				areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				Vector<String>tmp=new Vector<String>();
				tmp.add(nombreMensajesOriginales.elementAt(i));
				sleep(500);
				
				for (int j=0; j<nombreMensajesFalsos.size(); j++)
				{
					if (hashMensajesFalsos.get(nombreMensajesFalsos.elementAt(j)).equals(hashMensajesOriginales.get(nombreMensajesOriginales.elementAt(i))))
					{
						tmp.add(nombreMensajesFalsos.elementAt(j));
			    		StyleConstants.setBold(attrs, true);
						StyleConstants.setForeground(attrs, Color.blue);
						cadena = nombreMensajesFalsos.elementAt(j)+" - Hash: "+hashMensajesFalsos.get(nombreMensajesFalsos.elementAt(j))+"\n";
						areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);

						//muestro la generacion de hash en el mismo areaProceso
			    		StyleConstants.setBold(attrs, true);
			    		StyleConstants.setForeground(attrs, Color.red);
			    		cadena = "Hash: "+hashMensajesOriginales.get(nombreMensajesOriginales.elementAt(i))+"\n";

						StyleConstants.setBold(attrs, true);
			    		StyleConstants.setForeground(attrs, Color.blue);
			    		cadena = "Original: ";
		    			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena, attrs);
			    		StyleConstants.setForeground(attrs, Color.black);
			    		cadena = nombreMensajesOriginales.elementAt(i)+" ";
			    		areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena, attrs);
			    		StyleConstants.setForeground(attrs, Color.blue);
			    		cadena = "Falso: ";
		    			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena, attrs);
			    		StyleConstants.setForeground(attrs, Color.black);
			    		cadena = nombreMensajesFalsos.elementAt(j)+"\n";
		    			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena, attrs);
						sleep(800);
					}
					else
					{
			    		StyleConstants.setBold(attrs, false);
						StyleConstants.setForeground(attrs, Color.black);
						cadena = nombreMensajesFalsos.elementAt(j)+" - Hash: "+hashMensajesFalsos.get(nombreMensajesFalsos.elementAt(j))+"\n";
						areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
					}
	 				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
	    			areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());
				}
				if (tmp.size()>1) resultados.add(tmp);
				tmp = null;
				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
    			areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());
			}
		}		
		catch (BadLocationException ble)
    	{
    		ble.printStackTrace();
    	}
		catch (InterruptedException ie)
		{
			ie.printStackTrace();
		}
		
		StyleConstants.setBold(attrs, true);
		StyleConstants.setForeground(attrs, Color.black);
		try
		{
			cadena = "\nProceso finalizado.";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena, attrs);
	    	areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
		
		System.out.println("Total de mensajes originales con falsos: "+resultados.size());
		for (int i=0; i<resultados.size(); i++)
		{
			System.out.print(resultados.elementAt(i).elementAt(0));
			for (int j=1; j<resultados.elementAt(i).size(); j++)
			{		
				System.out.print(" "+resultados.elementAt(i).elementAt(j));
			}
			System.out.println();
		}
		if (resultados.size()>0) return true;
		else return false;
	}
	/**
	 * ActionListener que selecciona el archivo falso para abrirlo en una ventana
	 * y mostrar su contenido.
	 * @author Jésus Vidal Panalés
	 *
	 */
	private class SeleccionarFalso implements ActionListener
	{
		/**
		 * Método obligatorio para procesar el evento
		 */
		public void actionPerformed (ActionEvent e)
		{
			JComboBox jcb = (JComboBox) e.getSource();
			if (jcb.getComponentCount()>0)
			{
				archivo_falso = (String) jcb.getSelectedItem();
			}
		}
	}

	/**
	 * ActionListener que selecciona el archivo original para abrirlo en una ventana
	 * y mostrar su contenido. Además rellena el combobox de mensajes falsos que
	 * tienen el mismo hash que el mensaje original seleccionado.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class SeleccionarOriginal implements ActionListener
	{
		public void actionPerformed (ActionEvent e)
		{
			JComboBox jcb = (JComboBox) e.getSource();
			comboFalsos.removeAllItems();
			if (jcb.getComponentCount()>0)
			{
				int j = jcb.getSelectedIndex();
				archivo_original = (String) jcb.getSelectedItem();
				for (int i=1; i<resultados.elementAt(j).size(); i++)
				{
					comboFalsos.addItem(resultados.elementAt(j).elementAt(i));
				}
			}
		}
	}
	
	/**
	 * Método que actualiza el combobox añadiendo el nombre del archivo
	 * del mensaje original una vez se ha realizado el ataque y se han
	 * obtenido resultados
	 *
	 */
	private void actualizarComboBox ()
	{
		for (int i=0;i<resultados.size(); i++)
		{
			comboOriginales.addItem(resultados.elementAt(i).elementAt(0));
		}
	}
}
