import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.text.*;

/**
 * Clase que genera mensajes dado el mensaje con las palabras a las que
 * se les ha añadido palabras alternativas.
 * @author Jesús Vidal Panalés
 *
 */
public class Generador extends Thread
{
	private Vector<String> []elementos;
	private Vector<String> palabras;
	private String ruta = "";
	private JTextPane areaResultados;
	private JFrame generador;
	private SimpleAttributeSet attrs;
	
	/**
	 * Clase que interactúa con la clase Generador para mostrar el proceso
	 * de la generación de mensajes.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class MostrarGenerador extends Thread
	{
		/**
		 * ActionListener para Cerrar la ventana y liberarla de la memoria
		 * @author jesus
		 *
		 */
		public class Cerrar implements ActionListener
		{
			/**
			 * Método obligatorio para procesar el evento
			 */
			public void actionPerformed(ActionEvent e)
			{
				JButton cerrar = (JButton) e.getSource();
				JFrame jc = (JFrame)cerrar.getParent().getParent().getParent().getParent().getParent();
				jc.dispose();
			}
		}
		
		/**
		 * Constructor de la clase
		 *
		 */
		public MostrarGenerador() {}
		
		/**
		 * Método necesario para la ejecución del hilo
		 */
		public void run()
		{
			JLabel informacion = new JLabel("El proceso puede durar bastante tiempo, tenga paciencia.");
			informacion.setPreferredSize(new Dimension (600, 25));
			informacion.setSize(new Dimension (600, 25));
			informacion.setFont(new Font("Arial", Font.BOLD, 14));
			informacion.setForeground(Color.white);
			informacion.setHorizontalTextPosition(SwingConstants.CENTER);
			informacion.setHorizontalAlignment(SwingConstants.CENTER);
			
			JScrollPane scrollResultados = new JScrollPane(areaResultados);
			scrollResultados.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			scrollResultados.setPreferredSize(new Dimension(550, 300));
			scrollResultados.setBorder
	        (
	        		BorderFactory.createCompoundBorder
	        		(
	        				BorderFactory.createEmptyBorder(5,5,5,5),
	        				scrollResultados.getBorder()
	                )
	        );
			scrollResultados.setBackground(Color.black);

			ImageIcon botonCerrarNormal = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarNormal.png"));
			ImageIcon botonCerrarPulsado = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarPulsado.png"));
			Dimension boton = new Dimension (botonCerrarNormal.getIconWidth(), botonCerrarNormal.getIconHeight());

			JButton botonCerrar = new JButton();
			botonCerrar.setIcon(botonCerrarNormal);
			botonCerrar.addActionListener(new Cerrar());
			botonCerrar.setToolTipText("Cierra la ventana");
			botonCerrar.setPreferredSize(boton);
			botonCerrar.setPressedIcon(botonCerrarPulsado);
			botonCerrar.setBorderPainted(false);

			JPanel panel = new JPanel();
			panel.setPreferredSize(new Dimension (600,420));
			panel.add(informacion);
			panel.add(scrollResultados);
			panel.add(botonCerrar);
			panel.setBackground(Color.black);
			
			generador.setBackground(Color.black);
			generador.setTitle("Generador");
			generador.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			generador.setPreferredSize(new Dimension (600,420));
			generador.setSize(new Dimension (600,420));
			generador.setResizable(false);
			generador.getContentPane().add(panel);
			generador.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("images/iconoHash.png")).getImage());
			generador.pack();
			generador.setVisible(true);
		}
	}
	
	/**
	 * Constructor del generador de mensajes
	 * @param elem Las palabras alternativas a la original correspondiente
	 * @param ruta La ruta donde se almacenarán los mensajes generados
	 * @param p Las palabras originales del mensaje
	 */
	public Generador (Vector<String> []elem, String ruta, Vector<String> p)
	{
		this.elementos = elem;
		this.ruta = ruta;
		this.palabras = p;
		this.generador = new JFrame();
		this.areaResultados = new JTextPane();
		attrs = new SimpleAttributeSet();
		StyleConstants.setBold(attrs, true);
		try
		{
			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), "Procesando...\n\n", attrs);
			areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());;
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}

		MostrarGenerador mg = new MostrarGenerador();
		mg.start();
	}
	
	/**
	 * Método necesario para la ejecución del hilo
	 */
	public void run ()
	{
		generarMensajes();
	}
	
	/**
	 * Método que genera los mensajes con palabras alternativas a la palabra original
	 * del índice del vector
	 * @param mensajes Mensajes que se van a procesar para generar los mensajes alternativos
	 * @param palabras Las palabras alternativas para la palabra original de cada mensaje
	 * @param indice El índice del vector donde se encuentra la palabra original
	 * @return Un Vector con los mensajes modificados
	 */
	private Vector<Vector> generar (Vector<Vector> mensajes, Vector<String> palabras, int indice)
	{
		Vector<Vector> resultado = new Vector<Vector>();
		for (int i=0; i<palabras.size(); i++)
		{
			for (int j=0; j<mensajes.size(); j++)
			{
				Vector<String>tmp = (Vector<String>)mensajes.elementAt(j).clone();
				tmp.setElementAt(palabras.elementAt(i),indice);
				resultado.add(tmp);
			}
		}
		return resultado;
	}
	
	/**
	 * Método que genera los mensajes. Primero se procesa el mensaje original
	 * para la primera palabra con palabras alternativas, y el proceso es iterativo,
	 * por cada palabra con palabras alternativas, se procesa el mensaje original
	 * y los ya generados anteriormente.
	 *
	 */
	private void generarMensajes()
	{
		Vector<Vector> resultado = new Vector<Vector>(); //resultado con los mensajes generados
		Vector<Vector> tmp = new Vector<Vector>(); //resultado temporal con los mensajes generados
		tmp.add(palabras); //se añade el menesaje original
		resultado.add(palabras); //se añade el mensaje original
		String cadena = ""; //donde se almacena el mensaje
		String archivo = ""; //donde se almacena el nombre completo del archivo a guardar el mensaje generado
		String msg = "msg"; //nombre del archivo
		String ext = ".txt"; //extensión del archivo
		int indice = 0; //índice que se le concatena al nombre del archivo
		PrintWriter outputStream = null;

		try
		{
			sleep(500);
		}
		catch (InterruptedException ie)
		{
			ie.printStackTrace();
		}

		//primero guardamos el mensaje original
		for (int i = 0; i < palabras.size()-1; i++)
			cadena += palabras.elementAt(i) + " ";
		cadena += palabras.elementAt(palabras.size()-1);

		try
		{
			archivo = this.ruta + File.separator + msg + indice + ext;
			outputStream = new PrintWriter (new FileWriter(archivo));
			outputStream.println(cadena);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.red);
			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), msg+indice+ext+"\n", attrs);
			areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());;
			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena+"\n\n", attrs);
			areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());;
			indice++;
		}
		catch (IOException ioe)
		{
			ioe.printStackTrace();
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
		outputStream.close();
		
		cadena = "";
		
		//ahora procesamos para generar el resto de mensajes
		for (int i = 0; i < elementos.length; i++)
		{
			//si se tienen palabras alternativas para una original (el índice i, indica la posición de la palabra original)
			//se tienen tantos elementos como palabras originales, pero sólo algunas tendrán palabras alternativas
			//de ahí ver si el tamaño es mayor que cero para el elemento i
			if (elementos[i].size()>0)
			{
				//generamos los mensajes para el la palabra original y las alternativas
				tmp = generar(tmp, elementos[i], i);
				//guardamos los resultados de los mensajes resultados
				for (int j=0; j<tmp.size(); j++)
				{
					resultado.add(tmp.elementAt(j));

					//lo convertimos a una cadena
					for (int k = 0; k < tmp.elementAt(j).size()-1; k++)
					{
						cadena += tmp.elementAt(j).elementAt(k) + " "; 
					}
					cadena += tmp.elementAt(j).elementAt(tmp.elementAt(j).size()-1);
					//generamos el nombre del archivo y lo guardamos
					archivo = this.ruta + File.separator + msg + indice + ext;
					try
					{
						sleep(60);
						outputStream = new PrintWriter (new FileWriter(archivo));
						outputStream.println(cadena);
						StyleConstants.setBold(attrs, true);
						StyleConstants.setForeground(attrs, Color.red);
						areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), msg+indice+ext+"\n", attrs);
						areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());;
						StyleConstants.setBold(attrs, false);
						StyleConstants.setForeground(attrs, Color.black);
						areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), cadena+"\n\n", attrs);
						areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());;
					}
					catch (IOException ioe)
					{
						ioe.printStackTrace();
					}
					catch (BadLocationException ble)
					{
						ble.printStackTrace();
					}
					catch (InterruptedException ie)
					{
						ie.printStackTrace();
					}
					outputStream.close(); //cerramos el archivo para guardarlo
					cadena = "";
					indice++;
				}
				//para la proxima palabra original con alternativas, se le procesarán
				//el mensaje original y los ya procesados
				tmp = (Vector<Vector>)resultado.clone();
			}
		}

		StyleConstants.setBold(attrs, true);
		StyleConstants.setForeground(attrs, Color.black);
		try
		{
			areaResultados.getStyledDocument().insertString(areaResultados.getStyledDocument().getLength(), "\nProceso finalizado", attrs);
			areaResultados.setCaretPosition(areaResultados.getStyledDocument().getLength());;
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
		
		JOptionPane.showMessageDialog(null, "Se generaron " + Integer.toString(resultado.size()) + " cartas.", "Generador", JOptionPane.INFORMATION_MESSAGE);
	}
}
