public class Hash
{
	private SDES sdes;
	private String mensaje;
	private String preliminar;
	private int rondas;
	private String hash;
	
	/**
	 * Constructor de la clase Hash
	 * @param mensaje El mensaje de texto original
	 * @param r Las rondas para el proceso SDES
	 */
	public Hash (String mensaje, int r)
	{
		this.mensaje = texto_a_binario(mensaje);
		this.rondas = r;
	}
	
	/**
	 * Función compresor: Comprime 21 bits en 12 bits usando la función SDES
	 * @param cadena Mensaje de 21 bits
	 * @return Mensaje comprimido de 12 bits
	 */
	private String compresor (String cadena)
	{
		String tmp;
		tmp = cadena.substring(0, 12);
		String clave = cadena.substring(12);
		sdes = new SDES (tmp,this.rondas,clave);
		return sdes.algoritmoSDES();
	}
	
	/**
	 * Función que realiza los pasos anteriores a hacer el hash del mensaje.
	 * Prepara la cadena para que pueda ser procesada por la función hash.
	 * @param mensaje El mensaje original para ser procesado.
	 * @return El mensaje original procesado para poder realizar el hash.
	 */
	private String funcion_preliminar (String mensaje)
	{
		int r = 9;
		int resto1, resto2, longitud;
		String longitud_binario;
		
		longitud = this.mensaje.length();
		resto1 = longitud % r;

//		System.out.println ("Resto de longitud de mensaje con r: " + resto1);
					
		this.preliminar = this.mensaje;
		
		// Añado ceros a la izquierda para que sea entero divisible por r
		int j = r - resto1;
		for (int i = 0; i < j; i++)
			this.preliminar = '0' + this.preliminar;
		
//		System.out.println("El mensaje divisible por r: "+this.preliminar+ " longitud: "+this.preliminar.length());
		
		// Añado r ceros por la derecha
		for (int i = 0; i < r; i++)
			this.preliminar = this.preliminar + '0';

//		System.out.println("El mensaje con r ceros a la derecha: "+this.preliminar+" longitud: "+this.preliminar.length());
		
		// Ahora cojo la longitud inicial y la convierto a binario
		longitud_binario = Integer.toBinaryString(longitud);

//		System.out.println("Longitud binaria del mensaje: "+longitud_binario);
		
		resto2 = longitud_binario.length() % (r-1);
		
//		System.out.println("Longitud de mensaje modulo r-1: "+resto2 + " longitud: "+longitud_binario.length());

		// Añado ceros por la izquierda para que sea divisible por r-1
		int k =  (r-1) - resto2;
		for (int i = 0; i < k; i++)
			longitud_binario = '0' + longitud_binario;

//		System.out.println("Longitud divisible por r-1: "+longitud_binario);
		
		int l = longitud_binario.length() / (r-1);

		String cadena2 = "";
		int indice = 0;
		// Añado un 1 cada r-1 bits
		for (int i = 0; i < l; i++)
		{
			cadena2 = cadena2 + '1' + longitud_binario.substring(indice, indice + r-1);
			indice += r - 1;
		}

//		System.out.println("Cadena resultante con los 1 cada r-1: "+cadena2);
		
		// Concateno el mensaje inicial procesado con el de la longitud binaria procesada
		this.preliminar = this.preliminar + cadena2;

//		System.out.println ("Cadena final del proceso (cadena original procesada + longitud procesada): "+this.preliminar);
		
		return this.preliminar;
	}
	
	/**
	 * Método que realiza la función del hash.
	 * @return Devuelve el hash del mensaje.
	 */
	public String funcion_hash ()
	{
		int n=12, r=9;
		String H0 = "";
		String H = "";
		
		// Primero hago el paso preliminar de procesamiento del mensaje
		String resultado = this.funcion_preliminar(this.mensaje);
//		System.out.println("Resultado de la fase preliminar: "+resultado+ " longitud: "+resultado.length());
		
		// Creo H0 --> 000000000000 lo hago con bucle por posibles modificaciones de n
		for (int i=0; i < n; i++)
			H0 = H0 + '0';
		
//		System.out.println ("H0: "+H0);

		// El siguiente H tendrá H0 y concatendo con xi
		H = H0;

		// Las rondas para coger bloques de r de xi
		int rondas = resultado.length() / r;
		
		int indice=0;
		for (int i=1; i < rondas+1; i++)
		{
//			System.out.println ("indice: "+indice);
//			System.out.println("H: "+ H +" xi:"+ resultado.substring(indice, r+indice));
			// Concateno el Hi-1 con un bloque de xi
			H = H + resultado.substring(indice, r+indice);
			// Comprimo para tener Hi+1
			H = compresor (H);
//			System.out.println ("H"+i+": "+H);
			// Incremento el indice de bloques
			indice += r;
		}
		
		this.hash = H;
		
		return this.hash;
	}

	/**
	 * Convierte un mensaje de texto de caracteres a mensaje de texto en binario 
	 * @param m Mensaje de texto de caracteres
	 * @return Mensajde de texto en binario
	 */
	private String texto_a_binario (String m)
	{
		String binario = "";
		String []tmp = new String[m.length()];
		StringBuffer sb = new StringBuffer(m);

		for (int i = 0; i < sb.length(); i++)
		{
			tmp[i] = Integer.toBinaryString(sb.charAt(i));
			
			if ((Integer.toBinaryString(sb.charAt(i))).length() < 8)
			{
				int k = 8 - (Integer.toBinaryString(sb.charAt(i))).length();
				for (int j=0; j< k; j++)
					tmp[i] = "0" + tmp[i];
			}
		}
		for (int i=0; i<tmp.length; i++)
			binario = binario.concat(tmp[i]);
		
		return binario;
	}
}
