import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Clase que implementa el Hash y muestra el proceso en una ventana
 * @author Jesús Vidal Panalés
 *
 */
public class HashGrafico extends Thread 
{
	private SDES sdes;
	private String mensaje;
	private String preliminar;
	private int rondas;
	private String hash;
	private JTextPane areaProceso;
	private SimpleAttributeSet attrs;
	private String cadena = "";
	
	/**
	 * Hilo que crea una ventana para interactuar con el proceso Hash
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class ProcesoHash extends Thread
	{
		/**
		 * ActionListener para Cerrar la ventana y liberarla de la memoria.
		 * @author Jesús Vidal Panalés
		 *
		 */
		private class Cerrar implements ActionListener
		{
			public void actionPerformed(ActionEvent e)
			{
				JButton cerrar = (JButton) e.getSource();
				JFrame jc = (JFrame)cerrar.getParent().getParent().getParent().getParent().getParent();
				jc.dispose();
			}
		}

		/**
		 * Constructor del Hilo que muestra el proceso Hash
		 *
		 */
		public ProcesoHash(){}

		/**
		 * Método necesario para la ejecución del hilo
		 */
		public void run()
		{
			JFrame procesoHash = new JFrame();
			JLabel etiquetaProceso = new JLabel("Proceso Hash");
			etiquetaProceso.setForeground(Color.white);
			JScrollPane scrollProceso = new JScrollPane(areaProceso);
			scrollProceso.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
			scrollProceso.setPreferredSize(new Dimension(700, 350));
			scrollProceso.setBorder
	        (
	        		BorderFactory.createCompoundBorder
	        		(
	        				BorderFactory.createEmptyBorder(5,5,5,5),
	        				scrollProceso.getBorder()
	                )
	        );
			scrollProceso.setBackground(Color.black);

			ImageIcon botonCerrarNormal = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarNormal.png"));
			ImageIcon botonCerrarPulsado = new ImageIcon(getClass().getClassLoader().getResource("images/botonCerrarPulsado.png"));
			Dimension boton = new Dimension (botonCerrarNormal.getIconWidth(), botonCerrarNormal.getIconHeight());

			JButton botonCerrar = new JButton();
			botonCerrar.setIcon(botonCerrarNormal);
			botonCerrar.addActionListener(new Cerrar());
			botonCerrar.setToolTipText("Cierra la ventana");
			botonCerrar.setPreferredSize(boton);
			botonCerrar.setPressedIcon(botonCerrarPulsado);
			botonCerrar.setBorderPainted(false);
			
			JPanel panel = new JPanel();
			panel.setPreferredSize(new Dimension (750,460));
			panel.add(etiquetaProceso);
			panel.add(scrollProceso);
			panel.add(botonCerrar);
			panel.setBackground(Color.black);
			
			procesoHash.setBackground(Color.black);
			procesoHash.setTitle("Proceso Hash");
			procesoHash.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
			procesoHash.setPreferredSize(new Dimension (750,460));
			procesoHash.setSize(new Dimension (750,460));
			procesoHash.setResizable(false);
			procesoHash.getContentPane().add(panel);
			procesoHash.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("images/iconoHash.png")).getImage());
			procesoHash.pack();
			procesoHash.setVisible(true);
		}
	}
	
	/**
	 * Constructor de la clase Hash
	 * @param mensaje El mensaje de texto original
	 * @param r Las rondas para el proceso SDES
	 */
	public HashGrafico (String mensaje, int r)
	{
		this.mensaje = texto_a_binario(mensaje);
		this.rondas = r;
		this.areaProceso = new JTextPane();
		ProcesoHash ph = new ProcesoHash();
		ph.run();
		
		attrs = new SimpleAttributeSet();
		try
		{
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.red);
			cadena = "Mensaje de texto:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = mensaje+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.red);
			cadena = "Mensaje binario:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
			
			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = this.mensaje+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
	}

	/**
	 * Método necesario para la ejecución del hilo
	 */
	public void run()
	{
		funcion_hash();
	}
	/**
	 * Función compresor: Comprime 21 bits en 12 bits usando la función SDES
	 * @param cadena Mensaje de 21 bits
	 * @return Mensaje comprimido de 12 bits
	 */
	private String compresor (String cadena)
	{
		String tmp;
		tmp = cadena.substring(0, 12);
		String clave = cadena.substring(12);
		sdes = new SDES (tmp,this.rondas,clave);
		return sdes.algoritmoSDES();
	}
	
	/**
	 * Función que realiza los pasos anteriores a hacer el hash del mensaje.
	 * Prepara la cadena para que pueda ser procesada por la función hash.
	 * @param mensaje El mensaje original para ser procesado.
	 * @return El mensaje original procesado para poder realizar el hash.
	 */
	private String funcion_preliminar (String mensaje)
	{
		int r = 9;
		int resto1, resto2, longitud;
		this.preliminar = this.mensaje;
		
		String longitud_binario;
		
		longitud = this.mensaje.length();
		resto1 = longitud % r;

		try
		{
			sleep(500);
			cadena = "\nProceso preliminar:\n";
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.red);
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "Resto de la longitud del mensaje con r:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = resto1+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			// Añado ceros a la izquierda para que sea entero divisible por r
			int j = r - resto1;
			for (int i = 0; i < j; i++)
				this.preliminar = '0' + this.preliminar;
			
			sleep(60);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "El mensaje divisible por r (tras añadir ceros):\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = this.preliminar+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
			sleep(60);

			// Añado r ceros por la derecha
			for (int i = 0; i < r; i++)
				this.preliminar = this.preliminar + '0';
	
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "El mensaje con r ceros a la derecha:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = this.preliminar+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
			
			// Ahora cojo la longitud inicial y la convierto a binario
			longitud_binario = Integer.toBinaryString(longitud);
	
			sleep(60);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "Longitud binaria del mensaje:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = longitud_binario+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			resto2 = longitud_binario.length() % (r-1);
			
			// Añado ceros por la izquierda para que sea divisible por r-1
			int k =  (r-1) - resto2;
			for (int i = 0; i < k; i++)
				longitud_binario = '0' + longitud_binario;
	
			sleep(60);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "Resultado de añadir ceros a la izquierda de la longitud para que sea divisible por r-1:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = longitud_binario+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			int l = longitud_binario.length() / (r-1);
	
			String cadena2 = "";
			int indice = 0;
			// Añado un 1 cada r-1 bits
			for (int i = 0; i < l; i++)
			{
				cadena2 = cadena2 + '1' + longitud_binario.substring(indice, indice + r-1);
				indice += r - 1;
			}

			sleep(60);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "Resultado de añadir un 1 cada r-1 bits:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = cadena2+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			// Concateno el mensaje inicial procesado con el de la longitud binaria procesada
			this.preliminar = this.preliminar + cadena2;

			sleep(60);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "Cadena final resultante de concatenar la cadena original procesada y la longitud procesada: X\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = this.preliminar+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
			sleep(60);
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
		catch(InterruptedException ie)
		{
			ie.printStackTrace();
		}

		return this.preliminar;
	}
	
	/**
	 * Método que realiza la función del hash.
	 * @return Devuelve el hash del mensaje.
	 */
	public String funcion_hash ()
	{
		int n=12, r=9;
		String H0 = "";
		String H = "";
		
		// Primero hago el paso preliminar de procesamiento del mensaje
		String resultado = this.funcion_preliminar(this.mensaje);
		try
		{
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.red);
			cadena = "\nProceso Hash:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			// Creo H0 --> 000000000000 lo hago con bucle por posibles modificaciones de n
			for (int i=0; i < n; i++)
				H0 = H0 + '0';
		
			sleep(60);
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.blue);
			cadena = "H0:\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = H0+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
			sleep(60);

			// El siguiente H tendrá H0 y concatendo con xi
			H = H0;
	
			// Las rondas para coger bloques de r de xi
			int rondas = resultado.length() / r;

			int indice=0;
			for (int i=1; i < rondas+1; i++)
			{
				// Concateno el Hi-1 con un bloque de xi
				H = H + resultado.substring(indice, r+indice);
				StyleConstants.setBold(attrs, true);
				StyleConstants.setForeground(attrs, Color.blue);
				cadena = "H"+(i-1)+" concatenado X"+(i-1)+" :\n";
				areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
	
				StyleConstants.setBold(attrs, false);
				StyleConstants.setForeground(attrs, Color.black);
				cadena = H+"\n";
				areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

				sleep(60);

				// Comprimo para tener Hi+1
				H = compresor (H);
				StyleConstants.setBold(attrs, true);
				StyleConstants.setForeground(attrs, Color.blue);
				cadena = "H"+i+" tras la función compresor:\n";
				areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

				StyleConstants.setBold(attrs, false);
				StyleConstants.setForeground(attrs, Color.black);
				cadena = H+"\n";
				areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
				areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
				sleep(60);

				// Incremento el indice de bloques
				indice += r;
			}
			
			this.hash = H;
			StyleConstants.setBold(attrs, true);
			StyleConstants.setForeground(attrs, Color.red);
			cadena = "\nHASH: \n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());

			StyleConstants.setBold(attrs, false);
			StyleConstants.setForeground(attrs, Color.black);
			cadena = this.hash+"\n";
			areaProceso.getStyledDocument().insertString(areaProceso.getStyledDocument().getLength(), cadena, attrs);
			areaProceso.setCaretPosition(areaProceso.getStyledDocument().getLength());
		}
		catch (BadLocationException ble)
		{
			ble.printStackTrace();
		}
		catch(InterruptedException ie)
		{
			ie.printStackTrace();
		}
		
		return this.hash;
	}

	/**
	 * Convierte un mensaje de texto de caracteres a mensaje de texto en binario 
	 * @param m Mensaje de texto de caracteres
	 * @return Mensajde de texto en binario
	 */
	private String texto_a_binario (String m)
	{
		String binario = "";
		String []tmp = new String[m.length()];
		StringBuffer sb = new StringBuffer(m);

		for (int i = 0; i < sb.length(); i++)
		{
			tmp[i] = Integer.toBinaryString(sb.charAt(i));
			
			if ((Integer.toBinaryString(sb.charAt(i))).length() < 8)
			{
				int k = 8 - (Integer.toBinaryString(sb.charAt(i))).length();
				for (int j=0; j< k; j++)
					tmp[i] = "0" + tmp[i];
			}
		}
		for (int i=0; i<tmp.length; i++)
			binario = binario.concat(tmp[i]);
		
		return binario;
	}
}
