import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.UIManager;
import java.io.*;
import java.util.*;
/**
 * Interfaz gráfica del HASH.
 * Aplicación principal.
 * @author Jesús Vidal Panalés
 *
 */

public class HashUI extends JFrame
{
	
	/* Variables para la parte gráfica */
	private static final long serialVersionUID = 1;

	/* Panel Principal */
	private JPanel panelPrincipal = new JPanel();

	/* Panel Mensaje Original */
	private JPanel panelMensaje = new JPanel();
	private JLabel tituloMensaje = new JLabel();
	private JTextPane areaMensajeTexto = new JTextPane();
	private JScrollPane areaMensaje;
	private JLabel tituloHash = new JLabel();
	private JTextField hashMensaje = new JTextField(12);

	/* Panel Generador */
	private JPanel panelPrincipalGenerador = new JPanel();
	private JLabel tituloGenerador = new JLabel();
	private JTextField []palabras_originales;
	private JTextField []palabras_alternativas;
	private JPanel panelGenerador;
	private JScrollPane areaGenerador;
	private JLabel original = new JLabel();
	private JLabel alternativo = new JLabel();
	private JLabel listado = new JLabel();
	private JLabel campo_vacio = new JLabel("   ");
	private JComboBox []combo;

	/* Panel Central */
	private JPanel panelCentral = new JPanel();
	private JLabel etiquetaRondas = new JLabel();
	private JTextField numRondas = new JTextField (5);
	private JLabel etiquetaProcesoHash;
	private JCheckBox mostrarProcesoHash;
	
	/* Panel Botones*/
	private JPanel panelBotones = new JPanel();
	private ImageIcon botonAbrirNormal;
	private ImageIcon botonAbrirPulsado;
	private JButton botonAbrir = new JButton();
	private ImageIcon botonHashNormal;
	private ImageIcon botonHashPulsado;
	private JButton botonHash = new JButton();
	private ImageIcon botonExtraerNormal;
	private ImageIcon botonExtraerPulsado;
	private JButton botonExtraer = new JButton();
	private ImageIcon botonGenerarNormal;
	private ImageIcon botonGenerarPulsado;
	private JButton botonGenerar = new JButton();
	private ImageIcon botonAtacarNormal;
	private ImageIcon botonAtacarPulsado;
	private JButton botonAtacar = new JButton();
	private ImageIcon botonAddNormal;
	private ImageIcon botonAddPulsado;
	private JButton []botonAdd;
	private ImageIcon botonSalirNormal;
	private ImageIcon botonSalirPulsado;
	private JButton botonSalir = new JButton();
	
	private boolean procesoHash; //mostrar proceso hash o no
	private String mensaje = ""; //almacena el mensaje de texto
	private Hash hash; //clase hash, usado para realizar el hash
	private int rondas;//rondas para el SDES
	private Vector<String> []palabras_generador;//palabras para el generador de mensajes
	private String []palabras;//palabras originales del mensaje
	
	/**
	 * Clase que implementa que se salga del programa al pinchar en el botón Salir
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Salir implements ActionListener 
	{
		/**
		 * Método para salir del programa
		 */
		public void actionPerformed (ActionEvent e)
		{
			System.exit(0);
		}
	}
	
	/**
	 * Clase que selecciona todo el texto al pinchar en una caja de texto.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Seleccionado implements FocusListener
	{
		/**
		 * Selecciona el texto del componente "enfocado".
		 */
		public void focusGained (FocusEvent e)
		{
			JTextPane componente = (JTextPane) e.getSource();
			componente.selectAll();
		}
		public void focusLost (FocusEvent e) {}
	}
	
	/**
	 * Clase que sirve para que las rondas del cifrado/descifrado
	 * sean números.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Rondas implements KeyListener
	{
		public void keyTyped (KeyEvent e) {}
		public void keyPressed (KeyEvent e) {}
		public void keyReleased (KeyEvent e)
		{
			JTextField componente = (JTextField)e.getSource();
			String temp = "";
			char []rondas_tmp = new char [componente.getText().length()];
			rondas_tmp = componente.getText().toCharArray();

			/**
			 * Si no es un número, no se acepta (además de otras teclas...)
			 */
			if (e.getKeyChar()!='0' && e.getKeyChar()!='1' &&
				e.getKeyChar()!='2' && e.getKeyChar()!='3' &&
				e.getKeyChar()!='4' && e.getKeyChar()!='5' &&
				e.getKeyChar()!='6' && e.getKeyChar()!='7' &&
				e.getKeyChar()!='8' && e.getKeyChar()!='9' &&
				e.getKeyCode()!=KeyEvent.VK_BACK_SPACE && e.getKeyCode()!=KeyEvent.VK_TAB 
				&& e.getKeyCode()!=KeyEvent.VK_CONTROL && e.getKeyCode()!=KeyEvent.VK_COPY
				&& e.getKeyCode()!=KeyEvent.VK_ENTER && e.getKeyCode()!=KeyEvent.VK_PASTE 
				&& e.getKeyCode()!=KeyEvent.VK_INSERT && e.getKeyCode()!=KeyEvent.VK_HOME
				&& e.getKeyCode()!=KeyEvent.VK_DELETE && e.getKeyCode()!=KeyEvent.VK_END
				&& e.getKeyCode()!=KeyEvent.VK_LEFT && e.getKeyCode()!=KeyEvent.VK_RIGHT
				&& e.getKeyCode()!=KeyEvent.VK_SHIFT)
			
			{
				for (int i=0; i < rondas_tmp.length; i++)
				{
					if (rondas_tmp[i]=='0' || rondas_tmp[i]=='1' || rondas_tmp[i]=='2' ||
						rondas_tmp[i]=='3' || rondas_tmp[i]=='4' || rondas_tmp[i]=='5' ||
						rondas_tmp[i]=='6' || rondas_tmp[i]=='7' || rondas_tmp[i]=='8' ||
						rondas_tmp[i]=='9') 
							
							temp = temp + rondas_tmp[i];
				}
				componente.setText(temp);
			}
		}		
	}

	/**
	 * Método para verificar que no esté vacío el campo de rondas o no sean 0 rondas.
	 * @return true en caso de que estén correctas las condiciones, false en caso contrario. 
	 */
	public boolean comprobarRondas()
	{
		if (this.numRondas.getText().equals("") || this.numRondas.getText().equals("0")) return false;
		else return true;
	}

	/**
	 * Clase que al pinchar en Hash, realiza el hash del mensaje de entrada de texto
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class HashMensaje implements ActionListener
	{
		public void actionPerformed (ActionEvent e)
		{
			if (mensaje.compareTo("") == 0)
				mensaje = areaMensajeTexto.getText();
		    rondas = Integer.parseInt(numRondas.getText());

/*			if (mensaje.length()>200)
			{
				JOptionPane.showMessageDialog(null, "El archivo contiene más de 200 caracteres\nse cogerán los 200 primeros.", "Mensaje largo", JOptionPane.WARNING_MESSAGE);
				mensaje = mensaje.substring(0, 200);
			}
*/			
		    if (procesoHash)
		    {
		    	HashGrafico hg = new HashGrafico(mensaje,rondas);
		    	hg.start();
		    }
		    else
		    {
				hash = new Hash(mensaje, rondas);
				hashMensaje.setText(hash.funcion_hash());
		    }
			mensaje = "";
		}
	}

	/**
 	 * Clase que implementa el ActionListener del botón Abrir.
	 * Sirve para cargar el texto en el areaMensajeTexto de un archivo de texto.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Abrir implements ActionListener
	{
		public void actionPerformed (ActionEvent e)
		{
			String texto = "";
			BufferedReader archivo;
			File file;
			
			JFileChooser chooser = new JFileChooser();
	    	
		    int returnVal = chooser.showOpenDialog((Component)e.getSource());
		    
	    	if(returnVal == JFileChooser.APPROVE_OPTION)
		    {
		    	try
		    	{
		    		file = chooser.getSelectedFile().getAbsoluteFile();
		    		archivo = new BufferedReader(new FileReader(file));
		    		String linea = "";
					mensaje = "";

			    	while ((linea = archivo.readLine()) != null)
			    	{
			    		mensaje += linea;
			    		texto += linea + "\n";	
			    	}
			    	texto = texto.trim();

			    }
		    	catch (FileNotFoundException fnfe)
			    {
			    	JOptionPane.showMessageDialog(null, "No existe el archivo", "Error", JOptionPane.ERROR_MESSAGE);
			    }
			    catch (IOException ioe)
			    {
			    	JOptionPane.showMessageDialog(null, "No se pudo leer el archivo", "Error", JOptionPane.ERROR_MESSAGE);
			    }
			    areaMensajeTexto.setText(texto);
		    }
		}
	}
	
	/**
	 * Clase que implementa añadir la palabra alternativa a la original
	 * en el combobox.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Add implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			for (int i = 0; i < botonAdd.length; i++)
			{
				if ((e.getSource().equals(botonAdd[i]) && (!palabras_alternativas[i].getText().equals(""))))
				{
					combo[i].addItem(palabras_alternativas[i].getText());
					palabras_alternativas[i].setText("");
				}
			}
		}
	}
	
	/**
	 * Actualiza el panel cada vez que se extrae el texto de un mensaje
	 * para crear palabras sustitutas y poder generar cartas.
	 */
	private void actualizarPanel()
	{
		panelGenerador.removeAll();
		panelGenerador.add(this.original);
		panelGenerador.add(this.alternativo);
		panelGenerador.add(this.listado);
		panelGenerador.add(this.campo_vacio);
		this.panelGenerador.setPreferredSize(new Dimension(430, 285));
	}
	
	/**
	 * Clase que implementa extraer el etexto del areaMensajeTexto para
	 * poder crear palabras alternativas por cada palabra que se desee.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Extraer implements ActionListener
	{
		public void actionPerformed(ActionEvent e)
		{
			mensaje = areaMensajeTexto.getText();
			if (panelGenerador.getComponentCount()>3) actualizarPanel();
			palabras = areaMensajeTexto.getText().split(" ");
			palabras_originales = new JTextField[palabras.length];
			palabras_alternativas = new JTextField[palabras.length];
			palabras_generador = new Vector[palabras.length];
			combo = new JComboBox[palabras.length];
			botonAdd = new JButton[palabras.length];
			int height = panelGenerador.getPreferredSize().height;
			for (int i=0; i<palabras.length;i++)
			{
				palabras_originales[i] = new JTextField (palabras[i]);
				palabras_originales[i].setPreferredSize(new Dimension (130,25));
				palabras_alternativas[i] = new JTextField ();
				palabras_alternativas[i].setPreferredSize(new Dimension (130,25));
				combo[i] = new JComboBox();
				combo[i].setPreferredSize(new Dimension (130,25));
				botonAdd[i] = new JButton();
				botonAdd[i].setPreferredSize(new Dimension (10,10));
				botonAdd[i].setBorderPainted(false);
				botonAdd[i].setIcon(botonAddNormal);
				botonAdd[i].setPressedIcon(botonAddPulsado);
				botonAdd[i].addActionListener(new Add());
				panelGenerador.add(palabras_originales[i]);
				panelGenerador.add(palabras_alternativas[i]);
				panelGenerador.add(combo[i]);
				panelGenerador.add(botonAdd[i]);
			
				height += combo[i].getPreferredSize().height + 3;
			}
			panelGenerador.setPreferredSize(new Dimension (panelGenerador.getPreferredSize().width,height));
			panelGenerador.validate();
			areaGenerador.validate();
		}
	}
	
	/**
	 * Clase que crea la ventana del generador de mensajes.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Generar implements ActionListener
	{
		public void actionPerformed (ActionEvent e)
		{
			mensaje = areaMensajeTexto.getText();
			if (combo == null)
				JOptionPane.showMessageDialog(panelPrincipal, "No hay palabras para generar cartas.", "Error", JOptionPane.ERROR_MESSAGE);
			else
			{
				JFileChooser chooser = new JFileChooser();
		    	chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		    	chooser.setDialogTitle("Selecciona el directorio donde almacenar las cartas generadas");
			    int returnVal = chooser.showOpenDialog((Component)e.getSource());
			    
		    	if(returnVal == JFileChooser.APPROVE_OPTION)
			    {
		    		File ruta = chooser.getSelectedFile();
		    		for (int i = 0; i < combo.length; i++)
		    		{
		    			palabras_generador[i] = new Vector<String>();
		    			for (int j = 0; j < combo[i].getItemCount(); j++)
		    			{
		    				palabras_generador[i].addElement((String)combo[i].getItemAt(j));
		    			}
		    		}
		    		Vector<String>origen = new Vector<String>();
		    		for (int i=0; i<palabras.length; i++)
		    			origen.add(palabras[i]);
		    		Generador g = new Generador(palabras_generador, ruta.getAbsolutePath(), origen);
		    		g.start();
			    }
			}
		}
	}
	
	/**
	 * Clase que crea la ventana para seleccionar las rutas
	 * de los mensajes originales y falsos, y posteriormente
	 * realiza el ataque de cumpleaños
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Atacar implements ActionListener
	{
		public void actionPerformed (ActionEvent e)
		{
			new SeleccionarRutas(rondas);
		}
	}
	
	private class Mostrar implements ItemListener
	{
		public void itemStateChanged (ItemEvent e)
		{
			if (e.getStateChange() == ItemEvent.DESELECTED)
			{
		    	tituloHash.setVisible(true);
		    	hashMensaje.setVisible(true);
		    	panelMensaje.validate();
				procesoHash = false;
			}
			else
			{
		    	tituloHash.setVisible(false);
		    	hashMensaje.setVisible(false);
		    	panelMensaje.validate();
		    	procesoHash = true;
			}
		}
	}
	
	/**
	 * Constructor de la interfaz gráfica de HASH
	 *
	 */
	public HashUI ()
	{
		/* Mensaje */
		this.tituloMensaje.setText("Mensaje");
		this.tituloMensaje.setFont(new Font("Arial", Font.BOLD, 14));
		this.tituloMensaje.setForeground(Color.white);
		this.tituloHash.setText("Hash");
		this.tituloHash.setFont(new Font("Arial", Font.BOLD, 14));
		this.tituloHash.setForeground(Color.white);
		
		this.areaMensaje = new JScrollPane(this.areaMensajeTexto);
        this.areaMensaje.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        this.areaMensaje.setPreferredSize(new Dimension(350, 240));
        this.areaMensaje.setBorder
        (
        		BorderFactory.createCompoundBorder
        		(
        				BorderFactory.createEmptyBorder(5,5,5,5),
                        this.areaMensaje.getBorder()
                )
        );
        this.areaMensaje.setBackground(Color.black);
		this.areaMensajeTexto.addFocusListener(new Seleccionado());
		
		/* Generador de mensajes */
		this.tituloGenerador.setText("Generador de mensajes");
		this.tituloGenerador.setFont(new Font("Arial", Font.BOLD, 14));
		this.tituloGenerador.setForeground(Color.white);
		
		this.original.setText("Original:");
		this.original.setPreferredSize(new Dimension (130,25));
		this.original.setHorizontalTextPosition(SwingConstants.CENTER);
		this.original.setHorizontalAlignment(SwingConstants.CENTER);
		this.original.setBorder(BorderFactory.createLineBorder(Color.white));
		this.original.setForeground(Color.white);
		this.alternativo.setText("Sustituta:");
		this.alternativo.setPreferredSize(new Dimension (130,25));
		this.alternativo.setHorizontalTextPosition(SwingConstants.CENTER);
		this.alternativo.setHorizontalAlignment(SwingConstants.CENTER);
		this.alternativo.setBorder(BorderFactory.createLineBorder(Color.white));
		this.alternativo.setForeground(Color.white);
		this.listado.setText("Listado:");
		this.listado.setPreferredSize(new Dimension (130,25));
		this.listado.setHorizontalTextPosition(SwingConstants.CENTER);
		this.listado.setHorizontalAlignment(SwingConstants.CENTER);
		this.listado.setBorder(BorderFactory.createLineBorder(Color.white));
		this.listado.setForeground(Color.white);
		
		this.panelGenerador = new JPanel();
		this.panelGenerador.add(this.original);
		this.panelGenerador.add(this.alternativo);
		this.panelGenerador.add(this.listado);
		this.panelGenerador.add(this.campo_vacio);
		this.panelGenerador.setBackground(Color.black);
		this.panelGenerador.setPreferredSize(new Dimension(430, 285));
		this.areaGenerador = new JScrollPane(this.panelGenerador);
		this.areaGenerador.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
		this.areaGenerador.setPreferredSize(new Dimension(460, 285));
        this.areaGenerador.setBorder
        (
        		BorderFactory.createCompoundBorder
        		(
        				BorderFactory.createEmptyBorder(5,5,5,5),
                        this.areaGenerador.getBorder()
                )
        );
        this.areaGenerador.setBackground(Color.black);
		
		/* Rondas */
		this.etiquetaRondas.setText("Rondas SDES: ");
		this.etiquetaRondas.setForeground(Color.white);
		this.etiquetaRondas.setFont(new Font("Arial", Font.BOLD, 14));
		
		this.numRondas.setText("16");
		this.rondas = Integer.parseInt(this.numRondas.getText());
		this.numRondas.addKeyListener(new Rondas());
		
		/* Mostrar proceso */
		this.procesoHash = false;
		this.etiquetaProcesoHash = new JLabel("Mostrar proceso hash");
		this.etiquetaProcesoHash.setForeground(Color.white);
		this.etiquetaProcesoHash.setFont(new Font("Arial", Font.BOLD, 14));
		this.mostrarProcesoHash = new JCheckBox();
		this.mostrarProcesoHash.setPreferredSize(new Dimension (50,15));
		this.mostrarProcesoHash.setHorizontalAlignment(SwingConstants.LEFT);
		this.mostrarProcesoHash.setForeground(Color.white);
		this.mostrarProcesoHash.setBackground(Color.black);
		this.mostrarProcesoHash.setSelected(false);
		this.mostrarProcesoHash.addItemListener(new Mostrar());

		/* Botones */
		try
		{
			this.botonAbrirNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonAbrirNormal.png"));
			this.botonAbrirPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonAbrirPulsado.png"));
			Dimension boton = new Dimension (this.botonAbrirNormal.getIconWidth(),this.botonAbrirNormal.getIconHeight());
			
			this.botonAbrir.setIcon(botonAbrirNormal);
			this.botonAbrir.addActionListener(new Abrir());
			this.botonAbrir.setToolTipText("Abre un fichero");
			this.botonAbrir.setPreferredSize(boton);
			this.botonAbrir.setPressedIcon(botonAbrirPulsado);
			this.botonAbrir.setBorderPainted(false);
			
			this.botonHashNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonHashNormal.png"));
			this.botonHashPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonHashPulsado.png"));
			this.botonHash.setIcon(botonHashNormal);
			this.botonHash.addActionListener(new HashMensaje());
			this.botonHash.setToolTipText("Realiza el hash del mensaje");
			this.botonHash.setPreferredSize(boton);
			this.botonHash.setPressedIcon(botonHashPulsado);
			this.botonHash.setBorderPainted(false);
			
			this.botonExtraerNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonExtraerNormal.png"));
			this.botonExtraerPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonExtraerPulsado.png"));
			this.botonExtraer.setIcon(botonExtraerNormal);
			this.botonExtraer.addActionListener(new Extraer());
			this.botonExtraer.setToolTipText("Extrae las palabras del mensaje");
			this.botonExtraer.setPreferredSize(boton);
			this.botonExtraer.setPressedIcon(botonExtraerPulsado);
			this.botonExtraer.setBorderPainted(false);
	
			this.botonAddNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonAddNormal.png"));
			this.botonAddPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonAddPulsado.png"));
	
			this.botonGenerarNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonGenerarNormal.png"));
			this.botonGenerarPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonGenerarPulsado.png"));
			this.botonGenerar.setIcon(botonGenerarNormal);
			this.botonGenerar.addActionListener(new Generar());
			this.botonGenerar.setToolTipText("Genera cartas");
			this.botonGenerar.setPreferredSize(boton);
			this.botonGenerar.setPressedIcon(botonGenerarPulsado);
			this.botonGenerar.setBorderPainted(false);
	
			this.botonAtacarNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonAtacarNormal.png"));
			this.botonAtacarPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonAtacarPulsado.png"));
			this.botonAtacar.setIcon(botonAtacarNormal);
			this.botonAtacar.addActionListener(new Atacar());
			this.botonAtacar.setToolTipText("Realiza el ataque de cumpleaños");
			this.botonAtacar.setPreferredSize(boton);
			this.botonAtacar.setPressedIcon(botonAtacarPulsado);
			this.botonAtacar.setBorderPainted(false);
			
			this.botonSalirNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonSalirNormal.png"));
			this.botonSalirPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonSalirPulsado.png"));
			this.botonSalir.setIcon(botonSalirNormal);
			this.botonSalir.addActionListener(new Salir());
			this.botonSalir.setToolTipText("Sale del programa");
			this.botonSalir.setPreferredSize(boton);
			this.botonSalir.setPressedIcon(botonSalirPulsado);
			this.botonSalir.setBorderPainted(false);
		}
		catch (NullPointerException npe)
		{
			npe.printStackTrace();
		}
		/* LAYOUT PANEL MENSAJE */
		GridBagLayout gbl1 = new GridBagLayout();
		GridBagConstraints gbc1 = new GridBagConstraints();

		this.panelMensaje.setLayout(gbl1);
		gbc1.fill = GridBagConstraints.CENTER;
		
		gbc1.gridx = 0;
	    gbc1.gridy = 0;
	    gbc1.weightx = 1.0;
	    gbc1.weighty = 0.0;
	    gbc1.gridheight = 1;
	    gbc1.gridwidth = 2;
	    
	    gbc1.insets = new Insets (0,0,10,0);
	    
	    gbl1.setConstraints(this.tituloMensaje, gbc1);

	    gbc1.insets = new Insets (0,0,0,0);
	    gbc1.fill = GridBagConstraints.HORIZONTAL;
	    gbc1.gridheight = 1;
	    gbc1.gridwidth = 1;
	    gbc1.weightx = 1.0;
	    gbc1.weighty = 0.0;

	    gbc1.gridy = 1;
	    gbc1.gridx = 0;
	    gbl1.setConstraints(this.areaMensaje, gbc1);
	
	    gbc1.gridy = 2;
	    gbc1.weightx = 1.0;
	    gbc1.weighty = 0.0;
	    gbc1.gridheight = 1;
	    gbc1.gridwidth = 2;
	    gbc1.insets = new Insets (0,0,10,0);
		gbc1.fill = GridBagConstraints.CENTER;
	    
	    gbl1.setConstraints(this.tituloHash, gbc1);
	    
	    gbc1.fill = GridBagConstraints.HORIZONTAL;
	    gbc1.insets = new Insets (0,0,0,0);
	    gbc1.gridy = 3;
	    gbl1.setConstraints(this.hashMensaje, gbc1);
	    
	    this.panelMensaje.add(this.tituloMensaje);
	    this.panelMensaje.add(this.areaMensaje);
	    this.panelMensaje.add(this.tituloHash);
	    this.panelMensaje.add(this.hashMensaje);
	    
	    this.panelMensaje.setBackground(Color.black);
	    this.panelMensaje.setBorder(BorderFactory.createLineBorder(Color.white));
	    /* LAYOUT PANEL GENERADOR */
	    GridBagLayout gbl2 = new GridBagLayout();
		GridBagConstraints gbc2 = new GridBagConstraints();

		this.panelPrincipalGenerador.setLayout(gbl2);
		gbc2.fill = GridBagConstraints.CENTER;
		
		gbc2.gridx = 0;
	    gbc2.gridy = 0;
	    gbc2.weightx = 1.0;
	    gbc2.weighty = 0.0;
	    gbc2.gridheight = 1;
	    gbc2.gridwidth = 2;
	    
	    gbc2.insets = new Insets (0,0,10,0);
	    
	    gbl2.setConstraints(this.tituloGenerador, gbc2);

	    gbc2.insets = new Insets (0,0,0,0);
	    gbc2.fill = GridBagConstraints.HORIZONTAL;
	    gbc2.gridheight = 1;
	    gbc2.gridwidth = 1;
	    gbc2.weightx = 1.0;
	    gbc2.weighty = 0.0;

	    gbc2.gridy = 1;
	    gbc2.gridx = 0;

	    gbl2.setConstraints(this.areaGenerador, gbc2);
	    this.panelPrincipalGenerador.add(tituloGenerador);
	    this.panelPrincipalGenerador.add(areaGenerador);
	    this.panelPrincipalGenerador.setBackground(Color.black);
	    this.panelPrincipalGenerador.setBorder(BorderFactory.createLineBorder(Color.white));

	    /* LAYOUT PANEL PRINCIPAL */
	    this.panelPrincipal.add(this.panelMensaje);
	    this.panelPrincipal.add(this.panelPrincipalGenerador);
	    this.panelPrincipal.setBackground(Color.black);

	    /* LAYOUT PANEL CENTRAL */
	    this.panelCentral.setLayout(new FlowLayout(FlowLayout.CENTER));
	    this.panelCentral.add(this.etiquetaProcesoHash);
	    this.panelCentral.add(this.mostrarProcesoHash);
	    this.panelCentral.add(this.etiquetaRondas);
	    this.panelCentral.add(this.numRondas);
	    this.panelCentral.setBackground(Color.black);
	    
	    /* LAYOUT PANEL BOTONES */
	    this.panelBotones.setLayout(new FlowLayout(FlowLayout.CENTER));
	    this.panelBotones.add(this.botonAbrir);
	    this.panelBotones.add(this.botonHash);
	    this.panelBotones.add(this.botonExtraer);
	    this.panelBotones.add(this.botonGenerar);
	    this.panelBotones.add(this.botonAtacar);
	    this.panelBotones.add(this.botonSalir);
	    this.panelBotones.setBackground(Color.black);

	    /* LAYOUT FRAME */
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();

		this.getContentPane().setLayout(gbl);
		gbc.fill = GridBagConstraints.CENTER;
		
		gbc.gridx = 0;
	    gbc.gridy = 0;
	    gbc.weightx = 1.0;
	    gbc.weighty = 0.0;
	    gbc.gridheight = 1;
	    gbc.gridwidth = 1;
	    
	    gbc.insets = new Insets (0,0,0,0);
	    gbl.setConstraints(this.panelPrincipal, gbc);
	    gbc.gridy = 1;
	    gbc.insets = new Insets (20,0,0,0);
	    gbl.setConstraints(this.panelCentral, gbc);

	    gbc.insets = new Insets (20,0,0,0);
	    gbc.gridy = 2;
	    gbl.setConstraints(this.panelBotones, gbc);

	    this.getContentPane().add(this.panelPrincipal);
	    this.getContentPane().add(this.panelCentral);
	    this.getContentPane().add(this.panelBotones);

	    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		try
		{
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
			this.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("images/iconoHash.png")).getImage());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		this.getContentPane().setBackground(Color.BLACK);
		this.setTitle("HASH");
		this.setMinimumSize(new Dimension(700,520));
		this.setLocation((int)Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2-this.getSize().width/2,(int)Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2-this.getSize().height/2);
		this.setResizable(false);
		this.pack();
		this.setVisible(true);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Apéndice de método generado automáticamente
		new HashUI ();
	}

}
