import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 * Clase que crea una ventana para seleccionar las rutas de los mensajes
 * originales y falsos, y llamar al proceso del ataque de cumpleaños
 * @author Jesús Vidal Panalés
 *
 */
public class SeleccionarRutas extends JFrame
{
	private final static long serialVersionUID = 1;
	private JLabel original;
	private JLabel falso;
	private JTextField rutaOriginaĺ;
	private JTextField rutaFalso;
	private JPanel panel;
	private JButton botonExaminarOriginal;
	private JButton botonExaminarFalso;
	private JButton botonAceptar;
	private JButton botonCancelar;
	private File ruta_originales;
	private File ruta_falsos;
	private int rondas;
	
	/**
	 * ActionListener para seleccionar la ruta de los mensajes originales o falsos
	 * según el botón que se haya pulsado.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Examinar implements ActionListener
	{
		public void actionPerformed (ActionEvent e)
		{
			JFileChooser chooser = new JFileChooser();
			if (e.getSource().equals(botonExaminarOriginal))
			{
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			chooser.setDialogTitle("Selecciona el directorio de mensajes originales");
				int rutaOriginales = chooser.showOpenDialog((Component)e.getSource());
			if(rutaOriginales == JFileChooser.APPROVE_OPTION)
			{
				ruta_originales = chooser.getSelectedFile();
				rutaOriginaĺ.setText(ruta_originales.getAbsolutePath());
			}
			}
			else
			{
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			chooser.setDialogTitle("Selecciona el directorio de mensajes falsos");
				int rutaFalsos = chooser.showOpenDialog((Component)e.getSource());
				if(rutaFalsos == JFileChooser.APPROVE_OPTION)
			{
				ruta_falsos = chooser.getSelectedFile();
				rutaFalso.setText(ruta_falsos.getAbsolutePath());
			}
			}
		}
	}
	
	/**
	 * ActionListener que cierra la ventana sin realizar ningún proceso
	 * y la libera de la memoria.
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Cancelar implements ActionListener
	{
		/**
		 * Método obligatorio para procesar el evento
		 */
		public void actionPerformed (ActionEvent e)
		{
			JButton cancelar = (JButton) e.getSource();
			JFrame jc = (JFrame)cancelar.getParent().getParent().getParent().getParent().getParent();
			jc.dispose();
		}
	}
	
	/**
	 * ActionListener para ejecutar el proceso del ataque de cumpleaños siempre
	 * y cuando estén seleccionadas las rutas de los mensajes originales y falsos
	 * @author Jesús Vidal Panalés
	 *
	 */
	private class Aceptar implements ActionListener
	{
		/**
		 * Método obligatorio para procesar el evento
		 */
		public void actionPerformed(ActionEvent e)
		{
			if (ruta_originales!=null && ruta_falsos!=null)
			{
			JButton aceptar = (JButton) e.getSource();
			JFrame jc = (JFrame)aceptar.getParent().getParent().getParent().getParent().getParent();
			AtaqueHash ataque = new AtaqueHash (ruta_originales, ruta_falsos, rondas);
			ataque.start();
			jc.dispose();
			}
			else JOptionPane.showMessageDialog(null, "Necesitas elegir las rutas de los mensajes originales y falsos", "Fallo de rutas", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Constructor de la ventana en la que se seleccionan las rutas de mensajes
	 * originales y falsos para realizar el ataque de cumpleaños
	 * @param rondas Las rondas para el SDES
	 */
	public SeleccionarRutas (int rondas)
	{
		this.rondas = rondas;
		this.original = new JLabel("Ruta origen:");
		this.original.setFont(new Font("Arial", Font.BOLD, 14));
		this.original.setPreferredSize(new Dimension(90,15));
		this.original.setForeground(Color.white);

		this.falso = new JLabel("Ruta falso:");
		this.falso.setFont(new Font("Arial", Font.BOLD, 15));
		this.falso.setPreferredSize(new Dimension(90,10));
		this.falso.setForeground(Color.white);

		this.rutaOriginaĺ = new JTextField (15);
		this.rutaOriginaĺ.setEditable(false);
		this.rutaOriginaĺ.setDisabledTextColor(Color.black);
		this.rutaFalso = new JTextField (15);
		this.rutaFalso.setEditable(false);
		this.rutaFalso.setDisabledTextColor(Color.black);
		
		ImageIcon botonExaminarNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonExaminarNormal.png"));
		ImageIcon botonExaminarPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonExaminarPulsado.png"));
		Dimension boton = new Dimension (botonExaminarNormal.getIconWidth(),botonExaminarNormal.getIconHeight());
		
		this.botonExaminarOriginal = new JButton();
		this.botonExaminarOriginal.setIcon(botonExaminarNormal);
		this.botonExaminarOriginal.addActionListener(new Examinar());
		this.botonExaminarOriginal.setToolTipText("Selecciona la carpeta de mensajes originales");
		this.botonExaminarOriginal.setPreferredSize(boton);
		this.botonExaminarOriginal.setPressedIcon(botonExaminarPulsado);
		this.botonExaminarOriginal.setBorderPainted(false);

		this.botonExaminarFalso = new JButton();
		this.botonExaminarFalso.setIcon(botonExaminarNormal);
		this.botonExaminarFalso.addActionListener(new Examinar());
		this.botonExaminarFalso.setToolTipText("Selecciona la carpeta de mensajes falsos");
		this.botonExaminarFalso.setPreferredSize(boton);
		this.botonExaminarFalso.setPressedIcon(botonExaminarPulsado);
		this.botonExaminarFalso.setBorderPainted(false);
		
		ImageIcon botonAceptarNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonAceptarNormal.png"));
		ImageIcon botonAceptarPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonAceptarPulsado.png"));
		this.botonAceptar = new JButton();
		this.botonAceptar.setIcon(botonAceptarNormal);
		this.botonAceptar.addActionListener(new Aceptar());
		this.botonAceptar.setToolTipText("Aceptar las rutas");
		this.botonAceptar.setPreferredSize(boton);
		this.botonAceptar.setPressedIcon(botonAceptarPulsado);
		this.botonAceptar.setBorderPainted(false);
		
		ImageIcon botonCancelarNormal = new ImageIcon (getClass().getClassLoader().getResource("images/botonCancelarNormal.png"));
		ImageIcon botonCancelarPulsado = new ImageIcon (getClass().getClassLoader().getResource("images/botonCancelarPulsado.png"));
		this.botonCancelar = new JButton();
		this.botonCancelar.setIcon(botonCancelarNormal);
		this.botonCancelar.addActionListener(new Cancelar());
		this.botonCancelar.setToolTipText("Cierra la ventana");
		this.botonCancelar.setPreferredSize(boton);
		this.botonCancelar.setPressedIcon(botonCancelarPulsado);
		this.botonCancelar.setBorderPainted(false);
		
		this.panel = new JPanel();
		this.panel.setPreferredSize(new Dimension (390,150));
		this.panel.add(this.original);
		this.panel.add(this.rutaOriginaĺ);
		this.panel.add(this.botonExaminarOriginal);
		this.panel.add(this.falso);
		this.panel.add(this.rutaFalso);
		this.panel.add(this.botonExaminarFalso);
		this.panel.add(this.botonAceptar);
		this.panel.add(this.botonCancelar);
		this.panel.setBackground(Color.black);
		
		this.getContentPane().add(this.panel);
		
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		try
		{
			UIManager.setLookAndFeel(UIManager.getCrossPlatformLookAndFeelClassName());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
		
		this.setIconImage(new ImageIcon(getClass().getClassLoader().getResource("images/iconoHash.png")).getImage());
		this.getContentPane().setBackground(Color.BLACK);
		this.setTitle("Selector de rutas");
		this.setMinimumSize(new Dimension(390,150));
		this.setLocation((int)Toolkit.getDefaultToolkit().getScreenSize().getWidth()/2-this.getSize().width/2,(int)Toolkit.getDefaultToolkit().getScreenSize().getHeight()/2-this.getSize().height/2);
		this.setResizable(false);
		this.pack();
		this.setVisible(true);
	}
}
